//    RDS USB driver for PC software  
//
// =====================================================
//  Description
//    This file include main functions
//       of RDS USB driver for PC software 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 17/07/06    V1.0
//       create
//=======================================

#include <windows.h>
#include <math.h>

/* include driver usb for RDS application */
#include "usbInterface.h"
#include "rds_usb.h"
#include "RdsUsbDriver.h"
#include "RdsUsbMain.h"

/* local define */
#define USB_INTERFACE_SIZE_DATA 8  /* max size for USB iodata */
#define RDSUSB_REPLYDATA 4         /* max used data for USB reply */ 

#define MIN_FREQ_AF 876
#define MAX_FREQ_AF 1079

/* static data */
static unsigned char iodata[USB_INTERFACE_SIZE_DATA];



/* read  data  to board */
/*===================== */

void   RdsUsbReadPSRam(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;
   int indexData,indexDataIO;
   
   /* read PSName in RAM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_PS_1, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadPSRam sendUSBVendorCmd %d", result);
   } 
   else
   {
       indexDataIO=0;
       for (indexData=0; indexData<RDSUSB_REPLYDATA;indexData++)
       {
           Data[indexData]=iodata[indexDataIO++];
       }
       result=sendUSBVendorCmdIn(UsbRDSBoard, RDS_USB_READ_PS_2, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
       if (result < 0)
       {
          printf("Error RdsUsbReadPSRam (2) sendUSBVendorCmd %d", result);
       } 
       else
       {
          indexDataIO=0;
          for (indexData=RDSUSB_REPLYDATA; indexData<(2*RDSUSB_REPLYDATA);indexData++)
          {
              Data[indexData]=iodata[indexDataIO++];
          }
       }
   }
}


void   RdsUsbReadRTRam(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result=0;
    int indexData=0;
    int indexDataIO;
    int USBRequest;

    USBRequest=RDS_USB_READ_RT_0;
    while ( (result>=0) && (USBRequest != (RDS_USB_READ_RT_F +1)))
    {
        result=sendUSBVendorCmdIn( UsbRDSBoard, USBRequest, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
        if (result < 0)
        {
            printf("Error RdsUsbReadRTRam sendUSBVendorCmd %d", result);
        } 
        else
        {   
            for (indexDataIO=0; indexDataIO<RDSUSB_REPLYDATA;indexDataIO++)
            {
                /* filtering not displayable data */
                if (iodata[indexDataIO] >=' ')
                {
                    Data[indexData++]=iodata[indexDataIO];
                }
                else
                {
                   Data[indexData++]=' ';
                }
            }            
            USBRequest++;
        }
    }

}

void   RdsUsbReadPTY_TP_TA_MS_DI(usb_dev_handle  * UsbRDSBoard,char * Data,int SelectDest)
{
   int result;
   
   /* read PTY in RAM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, SelectDest, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadPTY_TP_TA_MS_DI sendUSBVendorCmd %d", result);
   } 
   else
   {
       sprintf(Data,"%d",iodata[0]);
   }
}



void   RdsUsbReadPSE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;
   int indexData,indexDataIO;
   
   /* read PSName in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_E2P_PS_1, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadPSE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       indexDataIO=0;
       for (indexData=0; indexData<RDSUSB_REPLYDATA;indexData++)
       {
           Data[indexData]=iodata[indexDataIO++];
       }
       result=sendUSBVendorCmdIn(UsbRDSBoard, RDS_USB_READ_E2P_PS_2, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
       if (result < 0)
       {
          printf("Error RdsUsbReadPSE2p (2) sendUSBVendorCmd %d", result);
       } 
       else
       {
          indexDataIO=0;
          for (indexData=RDSUSB_REPLYDATA; indexData<(2*RDSUSB_REPLYDATA);indexData++)
          {
              Data[indexData]=iodata[indexDataIO++];
          }
       }
   }
}

void   RdsUsbReadRTE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result=0;
    int indexData=0;
    int indexDataIO;
    int USBRequest;

    USBRequest=RDS_USB_READ_E2P_RT_0;
    while ( (result>=0) && (USBRequest != (RDS_USB_READ_E2P_RT_F +1)))
    {
        result=sendUSBVendorCmdIn( UsbRDSBoard, USBRequest, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
        if (result < 0)
        {
            printf("Error RdsUsbReadRTE2p sendUSBVendorCmd %d", result);
        } 
        else
        {   
            for (indexDataIO=0; indexDataIO<RDSUSB_REPLYDATA;indexDataIO++)
            {
                /* filtering not displayable data */
                if (iodata[indexDataIO] >=' ')
                {
                   Data[indexData++]=iodata[indexDataIO];
                }
                else
                {
                   Data[indexData++]=' ';
                }
            }            
            USBRequest++;
        }
    }

}

void   RdsUsbReadPIE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;
   
   /* read PICode in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_E2P_PICODE, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadPIE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       sprintf(Data,"0x%04X",((iodata[0]<<8)+iodata[1]));
   }

}

void   RdsUsbReadNAFE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;
   
   /* read NAF in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_E2P_NA, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadNAFE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       sprintf(Data,"%d",iodata[0]);
   }
}

void   RdsUsbReadAFE2p(usb_dev_handle  * UsbRDSBoard,char * Data,int indexAF)
{
   int result;
   float newAF;
   
   /* read NAF in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_E2P_AF, indexAF, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadAFE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       newAF= (iodata[0]-1+ MIN_FREQ_AF)/10.0 ;

       sprintf(Data,"%5.1f",newAF);
   }
}

void   RdsUsbRead2AE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;
   
   /* read 2A group in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_CONFIG_2A, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbRead2AE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       sprintf(Data,"%d",iodata[0]);
   }
}
void   RdsUsbReadSTE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;
   
   /* read 2A group in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_READ_CONFIG_STEREO, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbReadSTE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       sprintf(Data,"%d",iodata[0]);
   }
}




/* write data  to board */
/*===================== */

void   RdsUsbWritePSRam(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;

   
   /* write PSName in RAM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_UPDATE_PS_1, ((Data[0]<<8)+ Data[1]), ((Data[2]<<8)+ Data[3]), iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbWritePSRam sendUSBVendorCmd %d", result);
   } 
   else
   {
       result=sendUSBVendorCmdIn(UsbRDSBoard, RDS_USB_UPDATE_PS_2, ((Data[4]<<8)+ Data[5]), ((Data[6]<<8)+ Data[7]), iodata, USB_INTERFACE_SIZE_DATA);
       if (result < 0)
       {
          printf("Error RdsUsbWritePSRam (2) sendUSBVendorCmd %d", result);
       } 
   }
}


void   RdsUsbWriteRTRam(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result=0;
    int IndexChar=0;
    int USBRequest;
    char NewRT[RDSUSB_SIZE_RTXT+1]="\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r";

    /* read Radio Texte  */
    strcpy(NewRT,Data);
    
    /* test if RT lenght is smaller than maxsize --> fill ending with  \r  */
    if (strlen(NewRT) != RDSUSB_SIZE_RTXT)
    {
       for (IndexChar=strlen(NewRT); IndexChar <RDSUSB_SIZE_RTXT; IndexChar++)
       {
          NewRT[IndexChar]='\r';
       }
    }
    IndexChar=0;
    USBRequest=RDS_USB_UPDATE_RT_0;
    while ( (result>=0) && (USBRequest != (RDS_USB_UPDATE_RT_F +1)))
    {
        result=sendUSBVendorCmdIn( UsbRDSBoard, USBRequest, ((NewRT[IndexChar]<<8)+ NewRT[IndexChar+1]), ((NewRT[IndexChar+2]<<8)+ NewRT[IndexChar+3]), iodata, USB_INTERFACE_SIZE_DATA);
        if (result < 0)
        {
            printf("Error RdsUsbWriteRTRam sendUSBVendorCmd %d", result);
        } 
        else
        {   
            IndexChar+=4;
            USBRequest++;
        }
    }
}




void   RdsUsbWritePSE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
   int result;

   
   /* write PSName in EEPROM */
   result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_STORE_PS_1, ((Data[0]<<8)+ Data[1]), ((Data[2]<<8)+ Data[3]), iodata, USB_INTERFACE_SIZE_DATA);
   if (result < 0)
   {
       printf("Error RdsUsbWritePSE2p sendUSBVendorCmd %d", result);
   } 
   else
   {
       result=sendUSBVendorCmdIn(UsbRDSBoard, RDS_USB_STORE_PS_2, ((Data[4]<<8)+ Data[5]), ((Data[6]<<8)+ Data[7]), iodata, USB_INTERFACE_SIZE_DATA);
       if (result < 0)
       {
          printf("Error RdsUsbWritePSE2p (2) sendUSBVendorCmd %d", result);
       } 
   }
}


void   RdsUsbWriteRTE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result=0;
    int IndexChar=0;
    int USBRequest;
    char NewRT[RDSUSB_SIZE_RTXT+1]="\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r\r";

    /* read Radio Text  */
    strcpy(NewRT,Data);
    
    /* test if RT lenght is smaller than maxsize --> fill ending with  \r  */
    if (strlen(NewRT) != RDSUSB_SIZE_RTXT)
    {
       for (IndexChar=strlen(NewRT); IndexChar <RDSUSB_SIZE_RTXT; IndexChar++)
       {
          NewRT[IndexChar]='\r';
       }
    }
    IndexChar=0;
    
    USBRequest=RDS_USB_STORE_RT_0;
    while ( (result>=0) && (USBRequest != (RDS_USB_STORE_RT_F +1)))
    {
        result=sendUSBVendorCmdIn( UsbRDSBoard, USBRequest, ((NewRT[IndexChar]<<8)+ NewRT[IndexChar+1]), ((NewRT[IndexChar+2]<<8)+ NewRT[IndexChar+3]), iodata, USB_INTERFACE_SIZE_DATA);
        if (result < 0)
        {
            printf("Error sendUSBVendorCmd %d", result);
        } 
        else
        {   
            IndexChar+=4;
            USBRequest++;
        }
    }
}


void   RdsUsbWritePTY(usb_dev_handle  * UsbRDSBoard,char * Data,int SelectDest)
{
    int result;
    int newPTY;
    
    sscanf(Data,"%d",&newPTY);
    
    /* write PTY in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, SelectDest, newPTY, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWritePTY %d sendUSBVendorCmd %d",SelectDest, result);
    } 
}


void   RdsUsbWriteTP(usb_dev_handle  * UsbRDSBoard,char * Data,int SelectDest)
{
    int result;
    int newTP;
    
    sscanf(Data,"%d",&newTP);
    
    /* write TP in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, SelectDest, newTP, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWriteTP %d sendUSBVendorCmd %d",SelectDest, result);
    } 
}


void   RdsUsbWriteTA(usb_dev_handle  * UsbRDSBoard,char * Data,int SelectDest)
{
    int result;
    int newTA;
    
    sscanf(Data,"%d",&newTA);
    
    /* write TA in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, SelectDest, newTA, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWriteTA %d sendUSBVendorCmd %d",SelectDest, result);
    } 
}


void   RdsUsbWriteMS(usb_dev_handle  * UsbRDSBoard,char * Data,int SelectDest)
{
    int result;
    int newMS;
    
    sscanf(Data,"%d",&newMS);
    
    /* write MS in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, SelectDest, newMS, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWriteMS %d sendUSBVendorCmd %d",SelectDest, result);
    } 
}


void   RdsUsbWriteDI(usb_dev_handle  * UsbRDSBoard,char * Data,int SelectDest)
{
    int result;
    int newDI;
    
    sscanf(Data,"%d",&newDI);
    
    /* write DI in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, SelectDest, newDI, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWriteDIE2p sendUSBVendorCmd %d", result);
    } 
}




void   RdsUsbWritePIE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result;
    int NewPICode;
    
    sscanf(Data,"%X",&NewPICode);
    
    /* write PICOde in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_STORE_PICODE, NewPICode, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWritePIE2p sendUSBVendorCmd %d", result);
    } 
}

void   RdsUsbWriteNAFE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result;
    int newNAF;
    
    sscanf(Data,"%d",&newNAF);
    
    /* write NbAF in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_STORE_NA, newNAF, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWriteNAFE2p sendUSBVendorCmd %d", result);
    } 

}



void   RdsUsbWriteAFE2p(usb_dev_handle  * UsbRDSBoard,char * Data,int indexAF)
{
   int result;
   int NewAF;
   float NewAFFloat;
   
   sscanf(Data,"%f",&NewAFFloat);
   NewAFFloat*=10.0;
   if (fmod(NewAFFloat,10.0) >=0.5)
   {
      NewAF=(int)ceil(NewAFFloat);
   }
   else
   {
      NewAF=(int)floor(NewAFFloat);
   }
   if ( (NewAF >=MIN_FREQ_AF) && (NewAF <= MAX_FREQ_AF))
   {
      NewAF = NewAF - MIN_FREQ_AF+1;   /* 1st freq =1 for 87.6MHz */
      /* write AF in EEPROM */
      result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_STORE_AF, NewAF, indexAF, iodata, USB_INTERFACE_SIZE_DATA);
      if (result < 0)
      {
        printf("Error RdsUsbWriteAFE2p sendUSBVendorCmd %d", result);
      } 
   }
   else
   {
       printf("Error RdsUsbWriteAFE2p invalid freq");   
   }
}

void   RdsUsbWrite2AE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result;
    int new2A;
    
    sscanf(Data,"%d",&new2A);
    /* write 2A Group  in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_CONFIG_2A, new2A, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWrite2AE2p sendUSBVendorCmd %d", result);
    } 

}
void   RdsUsbWriteSTE2p(usb_dev_handle  * UsbRDSBoard,char * Data)
{
    int result;
    int newST;
    
    sscanf(Data,"%d",&newST);
    
    /* write stereo/mono in EEPROM */
    result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_CONFIG_STEREO, newST, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbWriteSTE2p sendUSBVendorCmd %d", result);
    } 

}


void   RdsUsbWriteDataEdit(usb_dev_handle  *UsbRDSBoard,int WriteCmd,char * Data)
{
  
   switch (WriteCmd)
   {
   	case RDS_USB_UPDATE_PS_1:
   	     RdsUsbWritePSRam(UsbRDSBoard,Data);
   		break;
   
   	case RDS_USB_UPDATE_RT_0:
   	     RdsUsbWriteRTRam(UsbRDSBoard,Data);
   		break;

   	case RDS_USB_UPDATE_PTY:
   	     RdsUsbWritePTY(UsbRDSBoard,Data,RDS_USB_UPDATE_PTY);
   		break;

   	case RDS_USB_UPDATE_TP:
   	     RdsUsbWriteTP(UsbRDSBoard,Data,RDS_USB_UPDATE_TP);
   		break;

   	case RDS_USB_UPDATE_TA:
   	     RdsUsbWriteTA(UsbRDSBoard,Data,RDS_USB_UPDATE_TA);
   		break;

   	case RDS_USB_UPDATE_MS:
   	     RdsUsbWriteMS(UsbRDSBoard,Data,RDS_USB_UPDATE_MS);
   		break;

   	case RDS_USB_UPDATE_DI:
   	     RdsUsbWriteDI(UsbRDSBoard,Data,RDS_USB_UPDATE_DI);
   		break;



   	case RDS_USB_STORE_PS_1:
   	     RdsUsbWritePSE2p(UsbRDSBoard,Data);
   		break;

   	case RDS_USB_STORE_RT_0:
   	     RdsUsbWriteRTE2p(UsbRDSBoard,Data);
   		break;

   	case RDS_USB_STORE_PTY:
   	     RdsUsbWritePTY(UsbRDSBoard,Data,RDS_USB_STORE_PTY);
   		break;

   	case RDS_USB_STORE_TP:
   	     RdsUsbWriteTP(UsbRDSBoard,Data,RDS_USB_STORE_TP);
   		break;

   	case RDS_USB_STORE_TA:
   	     RdsUsbWriteTA(UsbRDSBoard,Data,RDS_USB_STORE_TA);
   		break;

   	case RDS_USB_STORE_MS:
   	     RdsUsbWriteMS(UsbRDSBoard,Data,RDS_USB_STORE_MS);
   		break;

   	case RDS_USB_STORE_DI:
   	     RdsUsbWriteDI(UsbRDSBoard,Data,RDS_USB_STORE_DI);
   		break;

   	case RDS_USB_STORE_PICODE:
   	     RdsUsbWritePIE2p(UsbRDSBoard,Data);
   		break;
   
   	case RDS_USB_STORE_NA:
   	     RdsUsbWriteNAFE2p(UsbRDSBoard,Data);
   		break;
   
   	case RDS_USB_CONFIG_2A:
   	     RdsUsbWrite2AE2p(UsbRDSBoard,Data);
   		break;
   
   	case RDS_USB_CONFIG_STEREO:
   	     RdsUsbWriteSTE2p(UsbRDSBoard,Data);
   		break;
   
   	default:
        printf("Error RdsUsbWriteDataEdit invalid WriteCmd:%d",WriteCmd);
   }

}

void   RdsUsbResetE2p(usb_dev_handle  *UsbRDSBoard)
{
    int result;
    
    /* reset EEPROM  to default value */
    result=sendUSBVendorCmdIn( UsbRDSBoard, RDS_USB_RESET_EEPROM, 0, 0, iodata, USB_INTERFACE_SIZE_DATA);
    if (result < 0)
    {
      printf("Error RdsUsbResetE2p sendUSBVendorCmd %d", result);
    } 


}



