//    USB interface functions of RDS software  (PC software)
//
// =====================================================
//  Description
//    This file include interface functions
//       of RDS PC software using : AVR USB driver  by OBJECTIVE DEVELOPMENT Software GmbH 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 02/07/06    V1.0
//       derivated  from file :
/* Name: usb-libusb.c
 * Project: usbcalls library
 * Author: Christian Starkjohann
 * Creation Date: 2006-02-02
 * Tabsize: 4
 * Copyright: (c) 2006 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: Proprietary, free under certain conditions. See Documentation.
 * This Revision: $Id: usb-libusb.c 288 2007-03-29 16:34:42Z cs $
 */

//=======================================
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "usbInterface.h"
#include "usb.h"
#include "rds_usb.h"

//usb_dev_handle *usbIODevice; /* The usb device handle */
#define usbDevice   usb_dev_handle  /* use libusb's device structure */



//static int  usesReportIDs;

/* ------------------------------------------------------------------------- */

static int  usbGetStringAscii(usb_dev_handle *dev, int index, int langid, char *buf, int buflen)
{
char    buffer[256];
int     rval, i;

    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, buffer, sizeof(buffer), 1000)) < 0)
        return rval;
    if(buffer[1] != USB_DT_STRING)
        return 0;
    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];
    rval /= 2;
    /* lossy conversion to ISO Latin1 */
    for(i=1;i<rval;i++){
        if(i > buflen)  /* destination buffer overflow */
            break;
        buf[i-1] = buffer[2 * i];
        if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
            buf[i-1] = '?';
    }
    buf[i-1] = 0;
    return i-1;
}

//int usbOpenDevice(usbDevice_t **device, int vendor, char *vendorName, int product, char *productName, int _usesReportIDs)
int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USB_ERROR_NOTFOUND;
static int          didUsbInit = 0;

    if(!didUsbInit){
        usb_init();
        didUsbInit = 1;
    }
    usb_find_busses();
    usb_find_devices();
    for(bus=usb_get_busses(); bus; bus=bus->next){
        for(dev=bus->devices; dev; dev=dev->next){
            if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product){
                char    string[256];
                int     len;
                handle = usb_open(dev); /* we need to open the device in order to query strings */
                if(!handle){
                    errorCode = USB_ERROR_ACCESS;
                    fprintf(stderr, "Warning: cannot open USB device: %s\n", usb_strerror());
                    continue;
                }
                if(vendorName == NULL && productName == NULL){  /* name does not matter */
                    break;
                }
                /* now check whether the names match: */
                len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
                if(len < 0){
                    errorCode = USB_ERROR_IO;
                    fprintf(stderr, "Warning: cannot query manufacturer for device: %s\n", usb_strerror());
                }else{
                    errorCode = USB_ERROR_NOTFOUND;
                    /* fprintf(stderr, "seen device from vendor ->%s<-\n", string); */
                    if(strcmp(string, vendorName) == 0){
                        len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
                        if(len < 0){
                            errorCode = USB_ERROR_IO;
                            fprintf(stderr, "Warning: cannot query product for device: %s\n", usb_strerror());
                        }else{
                            errorCode = USB_ERROR_NOTFOUND;
                            /* fprintf(stderr, "seen product ->%s<-\n", string); */
                            if(strcmp(string, productName) == 0)
                                break;
                        }
                    }
                }
                usb_close(handle);
                handle = NULL;
            }
        }
        if(handle)
            break;
    }
    if(handle != NULL){
        int rval, retries = 3;
        if(usb_set_configuration(handle, 1)){
            fprintf(stderr, "Warning: could not set configuration: %s\n", usb_strerror());
        }
        /* now try to claim the interface and detach the kernel HID driver on
         * linux and other operating systems which support the call.
         */
        while((rval = usb_claim_interface(handle, 0)) != 0 && retries-- > 0){
#ifdef LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP
            if(usb_detach_kernel_driver_np(handle, 0) < 0){
                fprintf(stderr, "Warning: could not detach kernel HID driver: %s\n", usb_strerror());
            }
#endif
        }
        if(rval != 0)
            fprintf(stderr, "Warning: could not claim interface\n");
/* Continue anyway, even if we could not claim the interface. Control transfers
 * should still work.
 */
        errorCode = 0;
        *device = handle;
//        usesReportIDs = _usesReportIDs;
    }
    return errorCode;
}

/* ------------------------------------------------------------------------- */

void    usbCloseDevice(usb_dev_handle *device)
{
    if(device != NULL)
        usb_close(device);
}







/*
 * sendUSBVendorCmdIn
 *
 * This Function send the found USB Device a control message to the default EndPoint.
 * The Type of Request is a read (USB_ENDPOINT_IN) with a vendor command (USB_TYPE_VENDOR) to a Device EndPoint (USB_RECIP_DEVICE).
 * See UsbView.exe or TestLibUSB_Win.exe for Details.
 *
 * return 0 on succes or < 0 if error occurs
 *
 */
int sendUSBVendorCmdIn(usb_dev_handle *device,int request, int value, int index, void *replydata, int len)
{
	int result;

	result = usb_control_msg(device,                                           // USB Device Pointer
                             USB_ENDPOINT_IN | USB_TYPE_VENDOR | USB_RECIP_DEVICE,  // USB_DIR_IN | USB_TYPE_VENDOR | USB_RECIP_OTHER
                             request,                                               // request see USB documentation
                             value,                                                 // value see USB documentation
                             index,                                                 // index see USB documentation
                             (char*)replydata,                                      // buffer for the replay data
                             len,                                                   // len of the replay buffer
                             1000);                                                 // timeout in ms
	return result;
}

/*
 * sendUSBVendorCmdOut
 *
 * This Function send the found USB Device a control message to the default EndPoint.
 * The Type of Request is a read (USB_ENDPOINT_OUT) with a vendor command (USB_TYPE_VENDOR) to a Device EndPoint (USB_RECIP_DEVICE).
 * See UsbView.exe or TestLibUSB_Win.exe for Details.
 *
 * return 0 on succes or < 0 if error occurs
 *
 */
int sendUSBVendorCmdOut(usb_dev_handle*device, int request, int value, int index, void *replydata, int len)
{
	int result;

	result = usb_control_msg(device,                                           // USB Device Pointer
                             USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE,  // USB_DIR_IN | USB_TYPE_VENDOR | USB_RECIP_OTHER
                             request,                                               // request see USB documentation
                             value,                                                 // value see USB documentation
                             index,                                                 // index see USB documentation
                             (char*)replydata,                                      // buffer for the replay data
                             len,                                                   // len of the replay buffer
                             1000);                                                 // timeout in ms
	return result;
}

