//    USB header interface functions of RDS software  (PC software)
//
// =====================================================
//  Description
//    This file include header interface functions
//       of RDS PC software using : AVR USB driver  by OBJECTIVE DEVELOPMENT Software GmbH 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 02/07/06    V1.0
//       create:
//=======================================

#ifndef USBINTERFACE_H
#define USBINTERFACE_H

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "usb.h"

//usb_dev_handle *usbIODevice; /* The usb device handle */

#define USB_ERROR_NONE      0
#define USB_ERROR_ACCESS    1
#define USB_ERROR_NOTFOUND  2
#define USB_ERROR_BUSY      16
#define USB_ERROR_IO        5
/* These are the error codes which can be returned by functions of this
 * module.
 */

/* ------------------------------------------------------------------------ */

typedef struct usbDevice    usbDevice_t;
/* This type represents a USB device internally. Only opaque pointers to this
 * type are available outside the module implementation.
 */

/* ------------------------------------------------------------------------ */


//static int  usesReportIDs;

/* ------------------------------------------------------------------------- */

static int  usbGetStringAscii(usb_dev_handle *dev, int index, int langid, char *buf, int buflen);



int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName);
/* This function opens a USB device. 'vendor' and 'product' are the numeric
 * Vendor-ID and Product-ID of the device we want to open. If 'vendorName' and
 * 'productName' are both not NULL, only devices with matching manufacturer-
 * and product name strings are accepted. If the device uses report IDs,
 * 'usesReportIDs' must be set to a non-zero value.
 * Returns: If a matching device has been found, USB_ERROR_NONE is returned and
 * '*device' is set to an opaque pointer representing the device. The device
 * must be closed with usbCloseDevice(). If the device has not been found or
 * opening failed, an error code is returned.
 */

void    usbCloseDevice(usb_dev_handle *device);
/* Every device opened with usbOpenDevice() must be closed with this function.
 */
 
/* ------------------------------------------------------------------------- */

 
 

/*
 * sendUSBVendorCmdIn
 *
 * This Function send the found USB Device a control message to the default EndPoint.
 * The Type of Request is a read (USB_ENDPOINT_IN) with a vendor command (USB_TYPE_VENDOR) to a Device EndPoint (USB_RECIP_DEVICE).
 * See UsbView.exe or TestLibUSB_Win.exe for Details.
 *
 * return 0 on succes or < 0 if error occurs
 *
 */
int sendUSBVendorCmdIn(usb_dev_handle *device,int request, int value, int index, void *replydata, int len);


/*
 * sendUSBVendorCmdOut
 *
 * This Function send the found USB Device a control message to the default EndPoint.
 * The Type of Request is a read (USB_ENDPOINT_OUT) with a vendor command (USB_TYPE_VENDOR) to a Device EndPoint (USB_RECIP_DEVICE).
 * See UsbView.exe or TestLibUSB_Win.exe for Details.
 *
 * return 0 on succes or < 0 if error occurs
 *
 */
int sendUSBVendorCmdOut(usb_dev_handle *device, int request, int value, int index, void *replydata, int len);

#endif

